﻿Imports System.Drawing.Drawing2D

Public Class Form1
    ' Drawing constants.
    Private Const Xmin As Single = -0.1F
    Private Const Xmax As Single = 1.0F
    Private Const Ymin As Single = -10.0F
    Private Const Ymax As Single = 350.0F
    Private DrawingTransform, InverseTransform As Matrix

    Private HasSolution As Boolean = False
    Private BestA, BestB, BestC, BestD As Double
    Private Points As New List(Of PointF)()

    ' Make a drawing transformation.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim world_rect As New RectangleF(Xmin, Ymin, Xmax - Xmin, Ymax - Ymin)
        Dim pts() As PointF = _
        { _
            New PointF(0, picGraph.ClientSize.Height), _
            New PointF(picGraph.ClientSize.Width, picGraph.ClientSize.Height), _
            New PointF(0, 0) _
        }
        DrawingTransform = New Matrix(world_rect, pts)
        InverseTransform = DrawingTransform.Clone()
        InverseTransform.Invert()

        '' Start with some test data.
        'Points.Add(New PointF(0.2F, 138.8F))
        'Points.Add(New PointF(0.3F, 210.9F))
        'Points.Add(New PointF(0.4F, 260.2F))
        'Points.Add(New PointF(0.45F, 277.5F))
        'Points.Add(New PointF(0.5F, 294.1F))
        'Points.Add(New PointF(0.55F, 302.4F))
        'Points.Add(New PointF(0.6F, 311.4F))
        'Points.Add(New PointF(0.65F, 318.5F))
        'Points.Add(New PointF(0.7F, 326.4F))
        'Points.Add(New PointF(0.75F, 333.1F))
        'Points.Add(New PointF(0.8F, 336.7F))
        'Points.Add(New PointF(0.85F, 339.0F))
        'Points.Add(New PointF(0.9F, 342.2F))
        'Points.Add(New PointF(0.95F, 343.0F))

        ' Pretty up the equation.
        rchEquation.Select(9, 4)
        rchEquation.SelectionCharOffset = 10
        rchEquation.Select(13, 1)
        rchEquation.SelectionCharOffset = 20
    End Sub

    ' Save a new point.
    Private Sub picGraph_MouseClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picGraph.MouseClick
        ' Transform the point to world coordinates.
        Dim pts() As PointF = {New PointF(e.X, e.Y)}
        InverseTransform.TransformPoints(pts)
        ' Debug.WriteLine(pts(0).ToString()) 

        ' Save the point.
        Points.Add(pts(0))
        picGraph.Refresh()
    End Sub

    ' Draw the points and best fit curve.
    Private Sub picGraph_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles picGraph.Paint
        e.Graphics.SmoothingMode = SmoothingMode.AntiAlias

        ' Use the drawing transformation.
        e.Graphics.Transform = DrawingTransform

        ' Draw the axes.
        DrawAxes(e.Graphics)

        ' Draw the curve.
        If (HasSolution) Then
            Const Xstep As Double = 0.01
            Dim x0 As Double = Xmin
            Dim y0 As Double = CurveFunctions.F(Xmin, BestA, BestB, BestC, BestD)
            Using thin_pen As New Pen(Color.Blue, 0)
                For x As Double = Xmin + Xstep To Xmax Step Xstep
                    Dim y1 As Double = F(x, BestA, BestB, BestC, BestD)

                    Try
                        e.Graphics.DrawLine(thin_pen, _
                            CSng(x0), CSng(y0), CSng(x), CSng(y1))
                    Catch
                    End Try

                    x0 = x
                    y0 = y1
                    If (y0 > 100000000.0) Then Exit For
                Next x
            End Using
        End If

        ' Draw the points.
        Const dx As Single = (Xmax - Xmin) / 100
        Const dy As Single = (Ymax - Ymin) / 100
        Using thin_pen As New Pen(Color.Black, 0)
            For Each pt As PointF In Points
                e.Graphics.FillRectangle(Brushes.White, _
                    pt.X - dx, pt.Y - dy, 2 * dx, 2 * dy)
                e.Graphics.DrawRectangle(thin_pen, _
                    pt.X - dx, pt.Y - dy, 2 * dx, 2 * dy)
            Next pt
        End Using
    End Sub

    ' Draw the axes.
    Private Sub DrawAxes(ByVal gr As Graphics)
        Const xthick As Single = 0.025F
        Const ythick As Single = 10.0F
        Const xstep As Single = 0.25F
        Const ystep As Single = 20.0F

        Using thin_pen As New Pen(Color.Black, 0)
            gr.DrawLine(thin_pen, Xmin, 0, Xmax, 0)
            For x As Single = xstep To Xmax Step xstep
                gr.DrawLine(thin_pen, x, -ythick, x, ythick)
            Next x

            gr.DrawLine(thin_pen, 0, Ymin, 0, Ymax)
            For y As Single = ystep To Ymax Step ystep
                gr.DrawLine(thin_pen, -xthick, y, xthick, y)
            Next y
        End Using

        ' Draw axis labels.
        gr.ResetTransform()

        ' Transform the points where we need to draw.
        Dim pt_list As New List(Of PointF)()
        Dim pts() As PointF

        ' Y axis labels.
        pt_list = New List(Of PointF)()
        For y As Single = 100 To Ymax Step 100
            pt_list.Add(New PointF(Xmin, y))
        Next y
        pts = pt_list.ToArray()
        DrawingTransform.TransformPoints(pts)
        Using string_format As New StringFormat()
            string_format.Alignment = StringAlignment.Near
            string_format.LineAlignment = StringAlignment.Center
            Dim i As Integer = 0
            For y As Single = 100 To Ymax Step 100
                gr.DrawString(y.ToString(), Me.Font, Brushes.Black, _
                    pts(i).X, pts(i).Y, string_format)
                i += 1
            Next y
        End Using

        ' X axis labels.
        pt_list = New List(Of PointF)()
        For x As Single = xstep To Xmax Step xstep
            pt_list.Add(New PointF(x, 0))
        Next x
        pts = pt_list.ToArray()
        DrawingTransform.TransformPoints(pts)
        Using string_format As New StringFormat()
            string_format.Alignment = StringAlignment.Center
            string_format.LineAlignment = StringAlignment.Far
            Dim i As Integer = 0
            For x As Single = xstep To Xmax Step xstep
                gr.DrawString(x.ToString(), Me.Font, Brushes.Black, _
                    pts(i).X, pts(i).Y, string_format)
                i += 1
            Next x
        End Using

        ' Use the drawing transformation again.
        gr.Transform = DrawingTransform
    End Sub

    ' Clear the points.
    Private Sub btnClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClear.Click
        Points = New List(Of PointF)()
        HasSolution = False
        picGraph.Refresh()

        txtA.Clear()
        txtB.Clear()
        txtC.Clear()
        txtD.Clear()
        txtError.Clear()
        txtPercentError.Clear()
    End Sub

    ' Find parameters for a weibull curve fit.
    Private Sub btnFit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnFit.Click
        Me.Cursor = Cursors.WaitCursor
        txtA.Clear()
        txtB.Clear()
        txtC.Clear()
        txtD.Clear()
        txtError.Clear()
        txtPercentError.Clear()
        Application.DoEvents()
        Dim start_time As Date = Date.Now

        ' Find a good fit.
        ' The final parameter, num_steps, determines how many initial
        ' values are tested for a, b, c, and d. For example, if num_steps = 4,
        ' then a, b, c, and d each take on 4 values for 4 ^ 4 = 256 initial points.
        Dim err As Double = FindGoodFit( _
            Points, BestA, BestB, BestC, BestD, _
            5, 50)

        Dim stop_time As Date = Date.Now
        Dim elapsed As TimeSpan = stop_time - start_time
        Console.WriteLine("Time: " & _
            elapsed.TotalSeconds.ToString("0.00") & " seconds")

        txtA.Text = BestA.ToString()
        txtB.Text = BestB.ToString()
        txtC.Text = BestC.ToString()
        txtD.Text = BestD.ToString()

        ' Display the error.
        ShowError()

        ' We have a solution.
        HasSolution = True
        picGraph.Refresh()

        Me.Cursor = Cursors.Default
    End Sub

    ' Regraph with the given parameters.
    Private Sub btnGraph_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGraph.Click
        BestA = Double.Parse(txtA.Text)
        BestB = Double.Parse(txtB.Text)
        BestC = Double.Parse(txtC.Text)
        BestD = Double.Parse(txtD.Text)
        ShowError()
        picGraph.Refresh()
    End Sub

    ' Display the error.
    Private Sub ShowError()
        ' Get the error.
        Dim err As Double = Math.Sqrt(ErrorSquared( _
            Points, BestA, BestB, BestC, BestD))
        txtError.Text = err.ToString()

        ' Get the total of the function's values to
        ' calculate a percentage.
        Dim total As Double = 0
        For Each pt As PointF In Points
            total += F(pt.X, BestA, BestB, BestC, BestD)
        Next pt
        total = err / total * 100
        txtPercentError.Text = total.ToString()
    End Sub
End Class
